from ramonak.tokenizer import sent_tokenize, word_tokenize


def test_word_tokenize():
    assert word_tokenize("а я, аднак, быў там... але даволі даўно") == [
        "а",
        "я",
        ",",
        "аднак",
        ",",
        "быў",
        "там",
        "...",
        "але",
        "даволі",
        "даўно",
    ]


def test_word_tokenize_remove_punct():
    assert word_tokenize(
        "а я, аднак, быў там... але даволі даўно", remove_punct=True
    ) == [
        "а",
        "я",
        "аднак",
        "быў",
        "там",
        "але",
        "даволі",
        "даўно",
    ]


def test_sent_tokenize():
    assert sent_tokenize("""
                         Нарадзіўся ў засценку Акінчыцы Мінскага павета Мінскай губерні. 
                         Бацькі — Міхал Казіміравіч і Ганна Юр’еўна (народжаная Лёсік) з вёскі Мікалаеўшчына (цяпер за 12 км ад Стоўбцаў). 
                         З 13-ці іх дзяцей да сталага веку дажыла толькі 9
                         """) == [
        "Нарадзіўся ў засценку Акінчыцы Мінскага павета Мінскай губерні.",
        "Бацькі — Міхал Казіміравіч і Ганна Юр’еўна (народжаная Лёсік) з вёскі Мікалаеўшчына (цяпер за 12 км ад Стоўбцаў).",
        "З 13-ці іх дзяцей да сталага веку дажыла толькі 9",
    ]



import io
import json
import os
import zipfile
from pathlib import Path
from typing import Tuple

import requests
from tqdm import tqdm

from ramonak import PACKAGES_PATH

PACKAGES = json.load(
    open(Path(os.path.dirname(__file__), "nexus.json"), "r", encoding="utf8")
)


def fetch_unzip(zip_file_url: str, destination_dir: str) -> Path:
    Path(destination_dir).mkdir(exist_ok=True, parents=True)
    bio = io.BytesIO()

    response = requests.get(zip_file_url, stream=True)
    with tqdm.wrapattr(
        bio,
        "write",
        miniters=1,
        desc=zip_file_url.split("/")[-1],
        total=int(response.headers.get("content-length", 0)),
    ) as fout:
        for chunk in response.iter_content(chunk_size=4096):
            fout.write(chunk)

    z = zipfile.ZipFile(bio)
    z.extractall(destination_dir)


def package_exists(package_name: str) -> bool:
    package_dir = package_path(package_name)

    if package_dir.exists() and any(package_dir.iterdir()):
        return True
    else:
        return False


def package_basic_info(package: str) -> Tuple[str, str, str]:
    package_version = ""
    package_name = ""
    package_author = ""

    if package.count("/") == 2:
        package_author, package_name, package_version = package.split("/")
    elif package.count("/") == 1:
        package_author, package_name = package.split("/")
        package_version = sorted(
            PACKAGES[package_author][package_name].items(),
            key=lambda x: x[0],
            reverse=True,
        )[0][0]
    else:
        raise Exception(
            "Wrong package name. At least author and package name must be present"
        )

    return package_author, package_name, package_version


def require(wished_package: str) -> None:
    print(f"Required {wished_package}...", end=" ")

    if package_exists(wished_package):
        print("OK")
        return
    else:
        print("Downloading...")

    package_author, package_name, package_version = package_basic_info(wished_package)

    file_url = PACKAGES[package_author][package_name][package_version]

    fetch_unzip(
        file_url,
        Path(PACKAGES_PATH, package_author, package_name, str(package_version)),
    )

    print(
        f"The package '{package_author}/{package_name}/{package_version}' has been installed successfully"
    )


def package_path(package_name: str) -> Path:
    return Path(PACKAGES_PATH, *(str(i) for i in package_basic_info(package_name)))



from ramonak.packages.actions import package_path, require

from .base import Stemmer


class FlexionStatStemmer(Stemmer):
    def __init__(self):
        require("@alerus/stemdata")

        self.unchangeable_words = []
        self.flexions = []

        for word in open(
            package_path("@alerus/stemdata") / "unchangeable_words.txt",
            "r",
            encoding="utf8",
        ).readlines():
            word = word.strip()

            if word:
                self.unchangeable_words.append(word)

        for flexion in open(
            package_path("@alerus/stemdata") / "flexions.txt", "r", encoding="utf8"
        ).readlines():
            flexion = flexion.strip()

            if flexion:
                self.flexions.append(flexion)

    def stem_word(self, word: str) -> str:
        if word in self.unchangeable_words:
            return word

        word = FlexionStatStemmer.fix_lang_phenomenons(word)

        for flexion in self.flexions:
            if word.endswith(flexion):
                return word[: -len(flexion)]

        return word



import os
from pathlib import Path

from dotenv import dotenv_values

os.environ = {**os.environ, **dotenv_values(".env.dev"), **dotenv_values(".env.prod")}

RAMONAK_PATH = Path(
    os.environ.get(
        "RAMONAK_PATH", os.path.join(os.path.expanduser("~"), ".alerus", "ramonak")
    )
)

PACKAGES_PATH = Path(RAMONAK_PATH, "packages")

for path in (RAMONAK_PATH, PACKAGES_PATH):
    path.mkdir(parents=True, exist_ok=True)



import re
from typing import Iterable, Union

from ramonak.packages.actions import package_path, require

require("@alerus/stopwords")

STOP_WORDS = (
    (package_path("@alerus/stopwords") / "belarusian.txt")
    .read_text(encoding="utf8")
    .split("\n")
)


def clean_stop_words(data: Union[str, Iterable[str]]) -> Union[str, Iterable[str]]:
    if type(data) is str:
        for word in STOP_WORDS:
            data = re.sub(r"\b{}\b".format(word), "", data)

        data = re.sub(r" {2,}", " ", data)
        data = data.strip()
    else:
        word_list = []

        for data_word in data:
            if type(data_word) is not str:
                raise Exception(
                    "Wrong type: {}. Data must be str or an iterable with str".format(
                        type(data_word).__name__
                    )
                )

            if data_word not in STOP_WORDS:
                word_list.append(data_word)

        data = word_list

    return data



from ramonak.stemmer.base import Stemmer

fix_lang_phenomenons = Stemmer.fix_lang_phenomenons


def test_dz_ts_and_jvowel():
    assert fix_lang_phenomenons("савецізіраваны") == "саветызіраваны"


def test_dz_ts_jvowel_and_v():
    assert fix_lang_phenomenons("дзве") == fix_lang_phenomenons("двэ")
